package com.ZoS.RoyalFlush;

import com.ZoS.RoyalFlush.Game.IGame;
import com.ZoS.RoyalFlush.Input.Keyboard;
import com.ZoS.RoyalFlush.Input.MouseCursor;

//TODO: Rework the comments to reflect the new changes!

/**
 * Welcome to the RoyalFlush engine! Today I'm going to walk you though your first
 * game with the engine and show you how to get started. The first thing you want to do
 * is copy this file and rename it to your game name (MyGame.java). Rename the class to that
 * and if you want to change the package the class is in (com.YourCompany.MyGame). Once you
 * Have that set up you can begin making your game. I'll run you though the rest of the file.
 * 
 * This file has been modified so you can read it straight down like a book. This is to help
 * familiarize new users with the material. Feel free to change this file as needed.
 * 
 * @author jpdeathblade
 */

/**
 * The first thing you will notice is that this extends com.ZoS.RoyalFlush.Game.IGame.
 * Any game using using the Royal Flush engine must extend this file as it sets up
 * all the methods you will need for your game to run. 
 */
public final class DefaultApplication implements IGame {
	
	//This is a application variable. This houses the data for the engine. With this
	//you have access to all sorts of information about the game as it is running and
	//you can even change things like resolution and max frames per second.
	static RoyalFlushApplication application;
	
	/**
	 * This is the very first function to run when the game begins. Here we create
	 * a new RoyalFlushApplication object and pass it a BaseGame object. This lets
	 * us know about the engine, and the engine know about us. We also pass in the 
	 * String[] args because the engine is set up to accept different command-line
	 * arguments. You can accept your own as well here. Once everything is set we
	 * start the application.
	 * 
	 * @param args - Command Line Arguments.
	 */
	public static void main(String[] args) {
		application = new RoyalFlushApplication(new DefaultApplication(), args);
		
		//Not engine forced but you can process String[] args for passed in parameters.
		parseCommandLineArgs(args);
	
		application.startApplication(800, 600, false, false, "Deafult Game");
	}

	/**
	 * Now that the application has begun the first thing it is going to do is call your
	 * games Initialize function. Here is where you can load in the assets you will need to get
	 * your came up and running. Anything from textures to level data can be loaded here. Once you are
	 * done, return true to let the Royal Flush engine that everything went smoothly. If something
	 * bad happens and you need the game to turn off, returning false will close the game.
	 */
	@Override
	public boolean Initialize() {
		//TODO: Load saved/default preferences.
		//TODO: Change applications width, height, fullscreen mode, vsync, etc.
		//TODO: Load all your game assets here.
		
		return true;
	}
	
	/**
	 * Now that your game is loaded you will enter what is called a "Game Loop". This loop will
	 * run forever until you tell it to stop. The loop will run in order and when it is done you will
	 * have 1 frame. The engine, if left at the default settings, will run 60 frames every second.
	 * The first function in the game loop is Input. This is where you check the current state of 
	 * any key, mouse, or gamepad connected to the computer and handle them. For example, if the user
	 * is pressing the left arrow key you might want to move the player left. That would be done here.
	 * Again, as will all the other functions, if everything goes the way you want return true to
	 * let the engine know. Otherwise you can return false to end the game loop and the game.
	 */
	@Override
	public boolean Input(double _deltaTime, MouseCursor mouse, Keyboard _keyboard) {
		//TODO: Check your input devices for changes.
		
		//If the game is active check for input.
		if(application.isFocused()) {
					
		}
		
		return true;
	}

	/**
	 * After Input the next function called is Update. Here you will want to update all the objects
	 * in the game. Here is where you can change states, play animations, and do physics and collision.
	 * You should also cull objects if you are using a frustum. Just like all the other 
	 * functions, true means everything is okay.
	 */
	@Override
	public boolean Update(double _deltaTime) {
		//TODO: Update the game state.
		
		//If the game is active update your game code.
		if(application.isFocused()) {
							
		}
		
		return true;
	}

	/**
	 * Next in line is Render3D. This is where you would move your camera, set up your 3D models
	 * and render them.
	 */
	@Override
	public boolean Render3D(double _deltaTime) {
		//TODO: Tighten the graphics on level 3.
		
		//If the game is visible render your game.
		if(application.isVisible()) {
					
		}
		
		return true;
	}

	/**
	 * Next up, Render2D. This is where you would render your HUD elements or any overlays your
	 * game may have. Or if you are doing a 2D game, this is where all of your rendering code would
	 * go to draw the levels, characters, and enemies. After this function the game loop starts
	 * all over again with Input.
	 */
	@Override
	public boolean Render2D(double _deltaTime) {
		//TODO: Show the player how much health they have left.
		
		//If the game is visible render your game.
		if(application.isVisible()) {
					
		}
		
		return true;
	}
	
	/**
	 * If you exit the game, or you break out of the game loop (through a GUI element or by
	 * something failing) the game will call Shutdown. In this last function you should clean up
	 * anything you did in Initialize. Get rid of extra memory, shutdown other objects, or even save
	 * the players current state. Once this is done, no matter true or false returned, the game is
	 * over and the engine will shut down.
	 */
	@Override
	public boolean Shutdown() {
		//TODO: Bye Bye =(
		
		return true;
	}
	
	private enum CommandLineArgs {
		ExampleInt, //The valid command line arguments for DefaultGame
		NOVALUE; //To handle errors
		
		public static CommandLineArgs toCommand(String _string)
	    {
	        try {
	            return valueOf(_string);
	        } 
	        catch (Exception ex) {
	            return NOVALUE;
	        }
	    } 
	}
	
	private static void parseCommandLineArgs(String[] _args) {
		//Process String[] args for passed in parameters. You can set them up however
		// you wish but for this example all parameters have the prefix "--dg"
		// Like this: --dgExampleInt
		
		//First this we want to do is loop through each string...
		for(String s: _args) {
			
			//Take the string and split it so we get the key and the value.
			// --dgExampleInt=10 becomes "-dgExampleInt" and "10"
			String[] command = s.split("=");
			
			//There should be a key and a value. If there is more then that
			// (more then 1 '=') then the command is invalid.
			if(command.length == 2) {
				
				//Now split the key to see if its an engine command.
				// --dgExampleInt becomes "--dg" and "ExampleInt"
				String[] key = command[0].split("--dg");
				
				//Make sure the key split into 2 halves. If it did then it is a
				// valid command.
				if(key.length == 2) {

					//This is where you check the command. To add more commands add the
					// command to the CommandLineArgs enum. They are case sensitive.
					switch(CommandLineArgs.toCommand(key[1])) {
						case ExampleInt: {
							//Now that we have our command we need to parse the string value.
							//many formats can do this already but some of those throw exceptions.
							//We should check for these exceptions so the game doesn't crash.
							//You can also limit the inputs by putting a range on a number or
							//only accepting certain values (like color names red, blue, green, etc).
							try {
								@SuppressWarnings("unused") //just an example.
								int example = Integer.parseInt(command[1]);
								System.out.print("DG Command: " + key[1] + " set to value: " + command[1] + "\n");
							}
							catch(NumberFormatException e) {
								System.err.print("DG Command: " + key[1] + " cannot set to value: " + command[1] + "\n");
							}
							break;
						}
					
						default: {
							//If this is called the command doesn't exist in the CommandLineArgs enum
							System.err.print("DG Command: " + s + " is not supported.\n");
							break;
						}
					}
				}
				else {
					//The command failed to parse because of its format (too many '--dg')
					System.err.print("DG Command: " + s + " skipped. Not an engine command.\n");
				}
			}
			else {
				//The command failed to parse because of its format (too many '=')
				System.err.print("DG Command: " + s + " failed to parse.\n");
			}
		}
	}
}