package com.ZoS.RoyalFlush.GUI;

import org.lwjgl.LWJGLException;
import org.lwjgl.opengl.Display;
import org.lwjgl.opengl.DisplayMode;


public final class OGLDisplay {
	
	private Window window;
	private DisplayMode[] displayModes	= null;
	private boolean updateDisplay = false;
	
	public OGLDisplay(int _width, int _height, boolean _fullscreen, boolean _vsync, String _title) {
		window = new Window(_width, _height, _fullscreen, _vsync, _title);
	}
	
	public boolean Create() {
		if(!findAvailableDisplayModes()) {
			return false;
		}
		
		try {
			syncWindow();
			this.centerApplication();
			Display.create();
		} catch (LWJGLException e) {
			//TODO: Create error logging object
			System.err.print("RoyalFlush: Display failed to create.");
			return false;
		}
		
		return true;
	}
	
	//TODO: Better name for this function
	private void syncWindow() throws LWJGLException{
		DisplayMode displayMode = new DisplayMode(this.window.getWidth(),this.window.getHeight());
		Display.setDisplayMode(displayMode);
		Display.setLocation(this.window.getX(), this.window.getY());
		Display.setFullscreen(this.window.isFullscreen());
		Display.setVSyncEnabled(this.window.getVSycn());
		Display.setResizable(this.window.isResizable());
		Display.setTitle(this.window.getTitle());
		Display.setInitialBackground(0.0f, 0.0f, 0.0f);
	}
	
	public boolean Update() {
		Display.update();
		Display.sync(this.window.getMaxFPS());
		
		//TODO: Better name for this
		if(updateDisplay) {
			try {
				syncWindow();
			} catch (LWJGLException e) {
				//TODO: Create error logging object
				System.err.print("RoyalFlush: Display failed to update.");
				return false;
			}
			updateDisplay = false;
		}
		
		//TODO: Check previous to see if it was resize
		// - Allow the user to drag until they release the mouse
		// - Not sure why you would want this though...
		if(Display.wasResized()) {
			this.window.setWindowSize(Display.getWidth(), Display.getHeight());
			updateDisplay = true;
		}

		return true;
	}
	
	public boolean Shutdown() {
		Display.destroy();
		return true;
	}
	
	private boolean findAvailableDisplayModes() {
		try {
			displayModes = Display.getAvailableDisplayModes();
		} catch (LWJGLException e) {
			//TODO: Create error logging object
			System.err.print("RoyalFlush: Display failed to find available display modes.");
			return false;
		}

		//TODO: Organize the display modes by: Resolution, BPP, then Frequency lowest to highest
		// example
		// 800x600, 16, 0HZ
		// 800x600, 16, 60HZ
		// 800x600, 32, 0HZ
		// 1920x180, 16, 0HZ
		for (int i=0;i<displayModes.length;i++) {
		    DisplayMode current = displayModes[i];
		    System.out.println(current.getWidth() + "x" + current.getHeight() + "x" +
		                        current.getBitsPerPixel() + " " + current.getFrequency() + "Hz");
		}
		
		return true;
	}
	
	/**
	 * Centers the application to the users screen based on the current display size.
	 */
	public void centerApplication() {
		int x = (Display.getDesktopDisplayMode().getWidth() / 2) - (this.window.getWidth() / 2);
		int y = (Display.getDesktopDisplayMode().getHeight() / 2) - (this.window.getHeight() / 2);
		this.setLocation(x, y);
	}
	
	public Window getWindow() {
		return this.window;
	}
	
	public boolean isCloseRequested() {
		return Display.isCloseRequested();
	}
	
	public boolean isFocused() {
		return Display.isActive();
	}
	
	public boolean isVisible() {
		return Display.isVisible();
	}
	
	public void setWidth(int _width) {
		this.window.setWidth(_width);
		updateDisplay = true;
	}
	
	public void setHeight(int _height) {
		this.window.setHeight(_height);
		updateDisplay = true;
	}
	
	public void setWindowSize(int _width, int _height) {
		this.window.setWindowSize(_width, _height);
		updateDisplay = true;
	}
	
	public void setX(int _x) {
		this.window.setX(_x);
		Display.setLocation(_x, this.window.getY());
	}
	
	public void setY(int _y) {
		this.window.setY(_y);
		Display.setLocation(this.window.getX(), _y);
	}
	
	public void setLocation(int _x, int _y) {
		this.window.setX(_x);
		this.window.setY(_y);
		Display.setLocation(_x, _y);
	}
	
	public void setFullScreen(boolean _fullscreen) {
		this.window.setFullscreen(_fullscreen);
		updateDisplay = true;
	}
	
	public void toggleFullScreen() {
		this.window.toggleFullscreen();
		updateDisplay = true;
	}
	
	public void setVSync(boolean _vsync) {
		this.window.setVSync(_vsync);
		Display.setVSyncEnabled(_vsync);
	}
	
	public void toggleVSync() {
		this.window.toggleVSync();
		Display.setVSyncEnabled(this.window.getVSycn());
	}
	
	public void setResizable(boolean _resizable) {
		this.window.setResizable(_resizable);
		Display.setResizable(_resizable);
	}
	
	public void toggleResizable() {
		this.window.toggleResizable();
		Display.setResizable(this.window.isResizable());
	}
	
	public void setTitle(String _title) {
		this.window.setTitle(_title);
		Display.setTitle(_title);
	}
	
	public void setMaxFPS(int _maxFPS) {
		this.window.setMaxFPS(_maxFPS);
	}
}