package com.ZoS.RoyalFlush.Renderer;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;

import org.lwjgl.opengl.ARBFragmentShader;
import org.lwjgl.opengl.ARBShaderObjects;
import org.lwjgl.opengl.ARBVertexShader;
import org.lwjgl.opengl.GL11;
import org.newdawn.slick.util.ResourceLoader;

public class Shader {
	private int vertexShader = 0;
	private int fragmentShader = 0;
	
	private int shaderProgram = 0;
	private boolean isBound = false;
	
	//TODO: Create init function, nothing that can fail should be called in constructor
	public Shader(String _vertexShader, String _fragmentShader) {
		super();
		
		try {
			vertexShader = createShader(_vertexShader, ARBVertexShader.GL_VERTEX_SHADER_ARB);
			fragmentShader = createShader(_fragmentShader, ARBFragmentShader.GL_FRAGMENT_SHADER_ARB);
		}
		catch(Exception exc) {
			exc.printStackTrace();
			return;
		}
		finally {
			if(vertexShader == 0 || fragmentShader == 0) {
				return;
			}
		}
		
		createProgram();
	}
	
	private int createShader(String _file, int _shaderType) throws Exception {
		int shader = 0;
    	try {
	        shader = ARBShaderObjects.glCreateShaderObjectARB(_shaderType);
	        
	        if(shader == 0)
	        	return 0;
	        
	        ARBShaderObjects.glShaderSourceARB(shader, readFileAsString(_file));
	        ARBShaderObjects.glCompileShaderARB(shader);
	        
	        if (ARBShaderObjects.glGetObjectParameteriARB(shader, ARBShaderObjects.GL_OBJECT_COMPILE_STATUS_ARB) == GL11.GL_FALSE)
	            throw new RuntimeException("Error creating shader: " + getLogInfo(shader));
	        
	        return shader;
    	}
    	catch(Exception exc) {
    		ARBShaderObjects.glDeleteObjectARB(shader);
    		throw exc;
    	}
	}
	
	private void createProgram() {
		shaderProgram = ARBShaderObjects.glCreateProgramObjectARB();

		if(shaderProgram == 0)
			return;
		/*
		* if the vertex and fragment shaders setup sucessfully,
		* attach them to the shader program, link the sahder program
		* (into the GL context I suppose), and validate
		*/
		ARBShaderObjects.glAttachObjectARB(shaderProgram, vertexShader);
		ARBShaderObjects.glAttachObjectARB(shaderProgram, fragmentShader);

		ARBShaderObjects.glLinkProgramARB(shaderProgram);
		if (ARBShaderObjects.glGetObjectParameteriARB(shaderProgram, ARBShaderObjects.GL_OBJECT_LINK_STATUS_ARB) == GL11.GL_FALSE) {
			System.err.println(getLogInfo(shaderProgram));
			return;
		}
 
		ARBShaderObjects.glValidateProgramARB(shaderProgram);
		if (ARBShaderObjects.glGetObjectParameteriARB(shaderProgram, ARBShaderObjects.GL_OBJECT_VALIDATE_STATUS_ARB) == GL11.GL_FALSE) {
			System.err.println(getLogInfo(shaderProgram));
			return;
		}
	}
	
	private static String getLogInfo(int _obj) {
        return ARBShaderObjects.glGetInfoLogARB(_obj, ARBShaderObjects.glGetObjectParameteriARB(_obj, ARBShaderObjects.GL_OBJECT_INFO_LOG_LENGTH_ARB));
    }
	
	public void bind() {
		if(shaderProgram != 0 && !isBound) {
			ARBShaderObjects.glUseProgramObjectARB(shaderProgram);
			isBound = true;
		}
		else if(shaderProgram != 0 && isBound) {
			System.err.print("RoyalFlush: Shader.bind - Shader program " + shaderProgram + " already bound.\n");
		}
		else {
			System.err.print("RoyalFlush: Shader.bind - No texture has been loaded.\n");
		}
	}
	
	public void unbind() {
		if(shaderProgram != 0 && isBound) {
			ARBShaderObjects.glUseProgramObjectARB(0);
			isBound = false;
		}
		else if(shaderProgram != 0 && !isBound) {
			System.err.print("RoyalFlush: Shader.unbind - Shader program " + shaderProgram + " is NOT bound.\n");
		}
		else {
			System.err.print("RoyalFlush: Shader.unbind - No shader to unbind.\n");
		}
	}
	
	private String readFileAsString(String filename) throws Exception {
        StringBuilder source = new StringBuilder();

        Exception exception = null;
        InputStream stream = ResourceLoader.getResourceAsStream(filename);
        
        BufferedReader reader;
        try{
            reader = new BufferedReader(new InputStreamReader(stream,"UTF-8"));
            
            Exception innerExc= null;
            try {
            	String line;
                while((line = reader.readLine()) != null)
                    source.append(line).append('\n');
            }
            catch(Exception exc) {
            	exception = exc;
            }
            finally {
            	try {
            		reader.close();
            	}
            	catch(Exception exc) {
            		if(innerExc == null)
            			innerExc = exc;
            		else
            			exc.printStackTrace();
            	}
            }
            
            if(innerExc != null)
            	throw innerExc;
        }
        catch(Exception exc) {
        	exception = exc;
        }
        finally {
        	try {
        		stream.close();
        	}
        	catch(Exception exc) {
        		if(exception == null)
        			exception = exc;
        		else
					exc.printStackTrace();
        	}
        	
        	if(exception != null)
        		throw exception;
        }
        
        return source.toString();
    }
}